require 'snogmetrics'
require 'kissmetrics'

module Saucy
  module Kiss
    class Observer
      def initialize(api_key)
        @api_key = api_key
      end

      def plan_viewed(data)
        javascript_client(data).record('Viewed Plan', {
          'Plan Name' => data[:plan].name,
          'Plan Price' => data[:plan].price,
          'Is Trial Plan?' => data[:plan].trial
        })
      end

      def plan_list_viewed(data)
        javascript_client(data).record('Viewed Plan List')
      end

      def account_created(data)
        javascript_client(data).record('Signed Up', {
          'Plan Name' => data[:account].plan.name,
          'Plan Price' => data[:account].plan.price,
          'Is Trial Plan?' => data[:account].plan.trial
        })
      end

      def plan_upgraded(data)
        javascript_client(data).record('Upgraded', {
          'Previous Plan Name' => data[:from_plan].name,
          'Plan Name' => data[:to_plan].name,
          'Plan Price' => data[:to_plan].price,
          'Is Trial Plan?'=> data[:to_plan].trial,
        })
      end

      def plan_downgraded(data)
        javascript_client(data).record('Downgraded', {
          'Previous Plan Name' => data[:from_plan].name,
          'Plan Name' => data[:to_plan].name,
          'Plan Price' => data[:to_plan].price,
          'Is Trial Plan?'=> data[:to_plan].trial,
        })
      end

      def billed(data)
        http_client.record(account_identifier(data), 'Billed', {
          'Billing Description' => "Monthly billing for #{data[:account].plan.name} plan",
          'Billing Amount' => data[:account].plan.price
        })
      end

      def canceled(data)
        plan = data[:account].plan
        javascript_client(data).record('Canceled', {
          'Plan Name' => plan.name,
          'Plan Price' => plan.price,
          'Is Trial Plan?'=> plan.trial
        })
      end

      def activated(data)
        plan = data[:account].plan
        http_client.record(account_identifier(data), 'Activated', {
          'Plan Name' => plan.name,
          'Plan Price' => plan.price,
          'Is Trial Plan?'=> plan.trial
        })
      end

      private

      def javascript_client(data)
        Snogmetrics::KissmetricsApi.new(@api_key, data[:request].session, snogmetrics_output_strategy)
      end

      def snogmetrics_output_strategy
        case Rails.env.to_s
        when 'development'
          :console_log
        when 'test'
          :array
        else
          :live
        end
      end

      def account_identifier(data)
        "account-#{data[:account].id}"
      end

      def http_client
        Kissmetrics::HttpClient.new(@api_key)
      end
    end
  end
end
